{% extends "base.html" %}
{% load i18n range %}
{% block extra_head %}
{{wizard.form.media}}
{% endblock %}
{% block content %}
<h2>{{wizard_label}}</h2>
<ul id='form_path'>
  <li class='current'><a href='#'>{{current_step.form_label}}</a></li>
</ul>
{% if wizard.form.forms %}
<div class='form'>
<div class='top_button'><input type="submit" id="submit_form" value="{% trans "Validate" %}"/></div>
<table class='formset'>
  {%if wizard.form.non_form_errors%}<tr class='error'><th colspan='2'>{{wizard.form.non_form_errors}}</th></tr>{%endif%}
  {{ wizard.form.management_form }}
  {% for formsetform in wizard.form.forms %}
    {{ formsetform.as_table }}
  {% endfor %}
  <tr class='modify'><td colspan="2"><button name="formset_modify" value="{{wizard.steps.current}}">{% trans "Add/Modify" %}</button></td></tr></li>
</table>
{% else %}
<div class='form'>
  {{ wizard.form.as_p }}
</div>
{% endif %}
{% endblock %}
