{% extends "ishtar/wizard/confirm_wizard.html" %}
{% load i18n %}
{% block "extra_informations" %}

{% if current_object.members.count %}
<h3>{% trans "Associated persons" %}</h3>
<ul class='list'>
{% for person in current_object.members.all %}
  <li>{{person}} <a class='display_details_inline' href="#" onclick='load_window("{%url show-person person.pk ''%}")'><i class="fa fa-info-circle" aria-hidden="true"></i></a></li>{% endfor %}
</ul>
{% endif %}

{% if current_object.files.count %}
<h3>{% trans "Associated archaeological files" %}</h3>
<ul class='list'>
{% for file in current_object.files.all %}
  <li>{{file}} <a class='display_details_inline' href="#" onclick='load_window("{%url show-file file.pk ''%}")'><i class="fa fa-info-circle" aria-hidden="true"></i></a></li>{% endfor %}
</ul>
{% endif %}

{% if current_object.operator.count %}
<h3>{% trans "Operator of archaeological operations" %}</h3>
<ul class='list'>
{% for operation in current_object.operator.all %}
  <li>{{operation}} <a class='display_details_inline' href="#" onclick='load_window("{%url show-operation operation.pk ''%}")'><i class="fa fa-info-circle" aria-hidden="true"></i></a></li>{% endfor %}
</ul>
{% endif %}

{% if current_object.adminact_operator.count %}
<h3>{% trans "Adminact: operator of archaeological operations" %}</h3>
<ul class='list'>
{% for adminact in current_object.adminact_operator.all %}
  <li>{{adminact}} <a class='display_details_inline' href="#" onclick='load_window("{%url show-administrativeact adminact.pk ''%}")'><i class="fa fa-info-circle" aria-hidden="true"></i></a></li>{% endfor %}
</ul>
{% endif %}

{% endblock %}
