{% extends "ishtar/wizard/confirm_wizard.html" %}
{% load url from future %}
{% load i18n %}

{% block "warning_informations" %}
{% for file in numeric_reference_files %}
{% if forloop.first %}
<p class='alert'><i class="fa fa-exclamation-triangle" aria-hidden="true"></i> <label>{% trans "The following files have the same numeric index as this file:" %}</label></p>
<ul>
{% endif%}
    <li>{{file}} <a href='#' onclick='load_window("{% url 'show-file' file.pk '' %}", "{{model_name}}");' class='display_details'><i class="fa fa-info-circle" aria-hidden="true"></i></a></li>
{% if forloop.last %}
</ul>
<hr/>
{% endif %}
{% endfor %}
{% for file in similar_files %}
{% if forloop.first %}
<p class='alert'><i class="fa fa-exclamation-triangle" aria-hidden="true"></i> <label>{% trans "The following files are in the same town and have parcels in common with this file:" %}</label></p>
<ul>
{% endif%}
    <li>{{file}} <a href='#' onclick='load_window("{% url 'show-file' file.pk '' %}", "{{model_name}}");' class='display_details'><i class="fa fa-info-circle" aria-hidden="true"></i></a></li>
{% if forloop.last %}
</ul>
<hr/>
{% endif %}
{% endfor %}
{% endblock %}
