{% load i18n %}{% if item.external_id %}
<p class='window-refs external-id'>
    <small title="{% trans 'Internal ID' %}">
        <i class="fa fa-key" aria-hidden="true"></i>
        {{item.external_id|default:''}}
    </small>
</p>{% endif %}
