{% extends "base.html" %}
{% load i18n inline_formset verbose_names %}
{% load url from future %}
{% block content %}
<h2>{{page_name}}</h2>
<div class='form'>
    <h3>{{object}}</h3>
    <form action="." method="post">{% csrf_token %}
        <p>{% trans "The current items are linked to this import:" %}</p>
        {% for accessor, imported in object.get_all_imported %}
        {% ifchanged imported|verbose_model_name %}
        {% if forloop.counter %}</ul>{% endif %}
        <h4>{{imported|verbose_model_name}}</h4>
        <ul class='item-list'>
        {%endifchanged%}
            <li>{{imported}}</li>
        {% endfor %}
        </ul>
        <p>{% trans "All these items will be deleted with this import." %}</p>
        <strong>{% trans "Are you sure?" %}</strong>
        <input type="submit" value="{% trans "Yes" %}"/>
    </form>
    <p><a href='{% url 'current_imports' %}'>{% trans "Back" %}</a></p>
</div>
{% endblock %}
