{% extends "ishtar/wizard/default_wizard.html" %}
{% load i18n range inline_formset %}
{% block extra_head %}
{{wizard.form.media}}
{% endblock %}
{% block wizard_form %}
<form action="." method="post">{% csrf_token %}
<div class='form'>
{{ wizard.form.media }}
{{ wizard.management_form }}
  {{ wizard.form.management_form }}
{% if automatic_parcel_association %}<p class='alert'><i class="fa fa-exclamation-triangle" aria-hidden="true"></i> <label>{%trans "Existing parcels from the operation will be automatically added to the archaeological file." %}</label></p>{% endif %}
  {%if wizard.form.non_form_errors%}
<table class='formset'>
<tr class='error'><th colspan='2'>{{wizard.form.non_form_errors}}</th></tr>
</table>{%endif%}
{{wizard.form.selection_form}}
<table class='inline-table' id='parcel-table'>
  <tr>{% for field in wizard.form.forms.0 %}<th{% if not forloop.last %} rowspan='2'{% endif %}>{{ field.label_tag }}</th>{% endfor %}</tr>
  <tr><td>({% trans "all"%} <input type='checkbox' name='check-all' class='check-all'/>)</td></tr>
  {% inline_formset 'Parcels' wizard.form.forms False %}
</table>
{% if add_all %}<p><input type='checkbox' name='add_all_parcels' id='add_all_parcels'> <label for='add_all_parcels'>{% trans "Add all parcels from the archaeological file" %}</label></p>{% endif %}
<p><button name="formset_modify" value="{{wizard.steps.current}}">{% trans "Add/Modify" %}</button></p>
<input type="hidden" name="{{ step_field }}" value="{{ step0 }}" />
{{ previous_fields|safe }}
  {% block "footer" %}
  <div id="footer">
    {% block "validation_bar" %}
    {% include 'ishtar/wizard/validation_bar.html' %}
    {% endblock %}
    {% include 'ishtar/blocks/footer.html' %}
  </div>
  {% endblock %}
</div>
</form>
{% endblock %}
