div#footer{
    position: fixed;
    bottom: 0px;
    background-color: #CCC;
    width: 100%;
    left: 0px;
    z-index: 101;
}

div#validation-bar{
    padding-top: 5px;
    margin-bottom: 7px;
}

div#foot{
    font-size: 0.75em;
    text-align: center;
    background-color: #bbb;
    padding: 0.2em;
}

#reset_wizards{
    right:50px;
    position:absolute;
}
