{% load i18n link_to_window %}
{% if item.history_creator.ishtaruser.person %}
<li><label>{% trans "Creation" context "Sheet" %}</label>
    <span class='value'>
        {{item.history_creator.ishtaruser.person}}
        {{item.history_creator.ishtaruser.person|link_to_window}} <br/>
        <small><em>{{item.history_creation_date|date:"DATETIME_FORMAT"}}</em></small>
    </span>
</li>
{% endif %}
{% if item.history_creation_date != item.last_edition_date %}
<li><label>{% trans "Modification" context "Sheet" %}</label>
    <span class='value'>
        {{item.history_modifier.ishtaruser.person}}
        {{item.history_modifier.ishtaruser.person|link_to_window}} <br/>
        <small><em>{% firstof item.history_date|date:"DATETIME_FORMAT" item.history.all.0.history_date|date:"DATETIME_FORMAT" %}</em></small>
    </span>
</li>
{% endif %}
