<input class="area_widget" type="text"{{final_attrs|safe}}> {{unit}} (<span id="ha_{{id}}">0</span> ha)
<script type="text/javascript"><!--//
    function evaluate_{{safe_id}}(){
        value = parseFloat($("#{{id}}").val());
        if(!isNaN(value)){
            value = value/10000;
        } else {
            value = 0;
        }
        $("#ha_{{id}}").html(value);
    }
    $("#{{id}}").keyup(evaluate_{{safe_id}});
    $(document).ready(evaluate_{{safe_id}}());
//--></script>
