{% extends "admin/base_site.html" %}

{% block content %}

<form action="." method="post" enctype="multipart/form-data">
{% csrf_token %}
<table>
{{ csv_form }}
</table>
<input type="hidden" name="action" value="import_generic" />
<input type="submit" name="apply" value="Import" />
</form>

{% endblock %}
