{% extends "ishtar/sheet_source.html" %}
{% load i18n window_field window_header link_to_window %}

{% block head_title %}{% trans "Treatment source" %}{% endblock %}

{% block window_nav %}
{% window_nav item window_id 'show-treatmentsource' 'treatment_source_modify' %}
{% endblock %}

{% block related %}
{% trans "Related treatment" as related_item_label %}
{% field related_item_label item.owner '' item.owner|link_to_window %}
{% endblock %}
