{% extends "base.html" %}
{% load i18n inline_formset %}
{% block content %}
<h2>{{page_name}}</h2>
<div class='form'>
<form enctype="multipart/form-data" action="." method="post"{% if confirm %}
    onsubmit='return confirm("{{confirm}}");'{% endif %}>{% csrf_token %}
<table>
{{form}}
</table>
<input id="submit_form" type="submit" value="{% if submit_label %}{{submit_label}}{% else%}{% trans "Validate" %}{% endif %}"/>
</form>
</div>
{% endblock %}
