{% extends "base.html" %}
{% load url from future %}
{% load i18n %}

{% block content %}
<div class='info'>
<p>{% trans "You may now login with your username and password." %}</p>
{% if not user.is_authenticated %}
<a href="{% url "auth_login" %}">{% trans "Login now" %}</a>
{% else %}
<a href="/">{% trans "Home" %}</a>
{% endif %}
</div>
{% endblock %}
