{% extends "base.html" %}
{% load i18n %}

{% block content %}
<div class='form'>
<form method="post" action=".">{% csrf_token %}
<table id='login'>
  <caption>{%trans "Log in"%}</caption>
{{ form.as_table }}
  <tr class='submit'><td colspan='2'><input type="submit" value="{% trans 'Log in' %}" /></td></tr>
  <input type="hidden" name="next" value="{{ next }}" />
</table>
</form>
</div>
<div class='info'>
<p>{% trans "Forgot password?" %} <a href="{% url auth_password_reset %}">{% trans "Reset it" %}</a></p>
<p>{% trans "Not member?" %} <a href="{% url registration_register %}">{% trans "Register" %}</a></p>
</div>
{% endblock %}
