{% extends "base.html" %}
{% load i18n inline_formset %}
{% block content %}
<h2>{{page_name}}</h2>
<div class='form'>
<form enctype="multipart/form-data" action="." method="post"
    onsubmit="return confirm('Do you really want to delete this item?');">{% csrf_token %}
<table>
{{form}}
</table>
<input type="submit" value="{% trans "Delete" %}"/>
</form>
</div>
{% endblock %}
